// $Id: AcpiWidget.cpp,v 1.56 2014/02/10 10:41:08 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/


#include "AcpiWidget.h"
#include "AcpiWidget.moc"
#include "AcpiConfigurationDialog.h"

#include "BaseContextMenu.h"
#include "Color.h"
#include "CustomPixmap.h"
#include "Debug.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "ToolTipWidget.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QLayout>
#include <QPainter>
#include <QShortcut>
#include <QStringList>
#include <QTextStream>
#include <QRegExp>

namespace Acpi
{
    //__________________________________________________
    AcpiWidget::AcpiWidget( QWidget* parent ):
        MainWindow( parent )
    {

        Debug::Throw( "AcpiWidget::AcpiWidget.\n" );
        setWindowTitle( Util::windowTitle( tr( "Battery Monitor" ) ) );

        QHBoxLayout* hLayout( new QHBoxLayout() );
        setLayout( hLayout );
        hLayout->setSpacing(0);
        hLayout->setMargin(0);

        // label
        hLayout->addWidget( label_ = new QLabel( this ), 0, Qt::AlignVCenter );

        // progressbar
        hLayout->addWidget( progressBar_ = new QProgressBar( this ), 1, Qt::AlignVCenter );
        _progressBar().setMaximum( 100 );
        _progressBar().setFixedHeight( fontMetrics().lineSpacing() + 7 );

        // actions
        _installActions();
        _configurationAction().setText( tr( "Battery Monitor Settings..." ) );

        // connections
        connect( &acpi_, SIGNAL(updated()), this, SLOT(_update()) );

        _updateConfiguration();

    }

    //________________________________________________
    void AcpiWidget::_configuration( void )
    {

        Debug::Throw( "AcpiWidget::_configuration.\n" );

        AcpiConfigurationDialog dialog(0);
        _configureFromDialog( dialog );

    }

    //__________________________________________________________
    void AcpiWidget::_updateConfiguration( void )
    {
        Debug::Throw( "AcpiWidget::_updateConfiguration.\n" );

        // color
        colors_[Acpi::Normal] = XmlOptions::get().get<Base::Color>( "NORMAL_COLOR" );
        colors_[Acpi::Low] = XmlOptions::get().get<Base::Color>( "LOW_COLOR" );
        colors_[Acpi::Critical] = XmlOptions::get().get<Base::Color>( "CRITICAL_COLOR" );

        acpi_.setThreshold( Acpi::Low, XmlOptions::get().get<int>( "LOW_THRESHOLD" ) );
        acpi_.setThreshold( Acpi::Critical, XmlOptions::get().get<int>( "CRITICAL_THRESHOLD" ) );

        // label
        _showLabelAction().setChecked( XmlOptions::get().get<bool>( "SHOW_LABEL" ) );

        //! pixmap
        pixmaps_.clear();
        pixmaps_[Acpi::Charged] = CustomPixmap().find( IconNames::BatteryCharged ).scaled( IconSize( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );
        pixmaps_[Acpi::Charging] = CustomPixmap().find( IconNames::BatteryCharging ).scaled( IconSize( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );
        pixmaps_[Acpi::Discharging] = CustomPixmap().find( IconNames::BatteryDischarging ).scaled( IconSize( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );
        pixmaps_[Acpi::Unknown] = CustomPixmap().find( IconNames::BatteryUnknown ).scaled( IconSize( IconSize::Medium ), Qt::KeepAspectRatio, Qt::SmoothTransformation );

        //! parent class
        MainWindow::_updateConfiguration();

        // margins
        setContentsMargins( _margins().left(), _margins().top(), _margins().right(), _margins().bottom() );

        // adjust height to minimum
        resize( width(), minimumHeight() );

        _updateTimer().start( _refresh()*1000, this );
        acpi_.update();
        _update();

    }

    //______________________________________________________________________
    void AcpiWidget::_paint( QPaintDevice& device, const QRect& clip )
    {
        if( _label().isVisible() ) _render( device, _label(), clip );
        if( _progressBar().isVisible() ) _render( device, _progressBar(), clip );
    }

    //______________________________________________________________________
    void AcpiWidget::_installContextMenuActions( BaseContextMenu* menu )
    {
        Debug::Throw( "AcpiWidget::_installContextMenuActions.\n" );

        // create parent menu
        MainWindow::_installContextMenuActions( menu );

        // actions
        menu->insertSeparator( &_menuSeparator() );
        menu->insertAction( &_menuSeparator(), &_showLabelAction() );
        menu->insertAction( &_menuSeparator(), &_updateAction() );

    }

    //__________________________________________________
    void AcpiWidget::timerEvent( QTimerEvent *event )
    {

        if( event->timerId() == _updateTimer().timerId() ) acpi_.update();
        MainWindow::timerEvent( event );

    }

    //______________________________________________________________________
    void AcpiWidget::_toggleLabel( bool state )
    {
        Debug::Throw( "AcpiWidget::_toggleLabel.\n" );
        _label().setVisible( state );
        XmlOptions::get().set<bool>( "SHOW_LABEL", state );
    }

    //______________________________________________________________________
    bool AcpiWidget::_updateToolTip( void )
    {

        Debug::Throw( "AcpiWidget::_updateToolTip.\n" );

        _toolTipWidget().setPixmap( windowIcon().pixmap( IconSize( IconSize::Maximum ) ) );

        // html formated tooltip
        // title
        QString buffer;
        QTextStream what( &buffer );
        what << "<div align='center' style='border: 10px;'><b>" << windowTitle() << "</b></div>" << endl;
        what << endl;

        if( !acpi_.stateText().isEmpty() )
        { what << "<div align='center'  style='border: 10px;'>" << acpi_.stateText() << "</div>" << endl; }

        if( !acpi_.tooltipText().isEmpty() )
        { what << "<div align='center'  style='border: 10px;'>" << acpi_.tooltipText() << "</div>" << endl; }

        _toolTipWidget().setText( buffer );
        return _toolTipWidget().isHidden();

    }

    //__________________________________________________
    void AcpiWidget::_installActions( void )
    {

        Debug::Throw( "AcpiWidget::_installActions.\n" );
        addAction( updateAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Update" ), this ) );
        _updateAction().setShortcut( Qt::CTRL + Qt::Key_U );
        _updateAction().setToolTip( tr( "Update" ) );
        connect( &_updateAction(), SIGNAL(triggered()), &acpi_, SLOT(update()));

        addAction( showLabelAction_ = new QAction( tr( "Show Status" ), this ) );
        _showLabelAction().setToolTip( tr( "Show/hide status label" ) );
        _showLabelAction().setCheckable( true );
        _showLabelAction().setChecked( true );
        connect( &_showLabelAction(), SIGNAL(toggled(bool)), SLOT(_toggleLabel(bool)));

    }

    //________________________________________________
    void AcpiWidget::_render( QPaintDevice& device, QWidget& widget, const QRect& clip )
    { widget.render( &device, widget.geometry().topLeft(), widget.rect().intersected( clip.translated( -widget.geometry().topLeft() ) ), 0 ); }

    //_________________________________________________________
    void AcpiWidget::_update( void )
    {

        Debug::Throw( "AcpiWidget::_update.\n" );

        QPalette palette;
        QColor color( colors_[acpi_.level()] );
        if( color.isValid() ) palette.setColor( QPalette::Highlight, color );
        _progressBar().setPalette( palette );

        _label().setPixmap( pixmaps_[acpi_.state()] );
        _label().update();

        _progressBar().setValue( acpi_.progress() );

        update();

        return;

    }

}
