#ifndef CalendarWidget_h
#define CalendarWidget_h
// $Id: CalendarWidget.h,v 1.49 2014/02/20 17:54:47 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CalendarCell.h"
#include "CalendarPath.h"
#include "CalendarEventList.h"
#include "MainWindow.h"
#include "RoundedPath.h"

#include <QBasicTimer>
#include <QDateTime>
#include <QTimerEvent>
#include <QPolygonF>
#include <QAction>
#include <QResizeEvent>

//! Calendar widgets
namespace Calendar
{

    //! forward declaration
    class CalendarHeader;

    //! transparent Calendar widget
    class CalendarWidget: public MainWindow
    {

        //! Qt meta object declaration
        Q_OBJECT

        public:

        //! constructor
        CalendarWidget( QWidget *parent = 0 );

        protected Q_SLOTS:

        //! header colors
        void _updateHeaderColors( void );

        protected:

        //!@name event handlers
        //@{

        //! generic event handler
        virtual bool event( QEvent* );

        //! mouse press event [overloaded]
        virtual void mousePressEvent( QMouseEvent* );

        //! mouse move event
        /*! used to track date matching pointer */
        virtual void mouseMoveEvent( QMouseEvent* );

        //! context menu event
        virtual void contextMenuEvent( QContextMenuEvent* );

        //! paint event handler
        virtual void _paint( QPaintDevice&, const QRect& );

        //! timer event
        virtual void timerEvent( QTimerEvent* );

        //! resize event [overloaded]
        virtual void resizeEvent( QResizeEvent* );

        //@}

        //! context menu
        virtual void _installContextMenuActions( BaseContextMenu* );

        //!@name current events
        //@{

        //! store list of current month events
        void _updateCurrentEvents( void );

        //! current events
        const CalendarEvent::List& _currentEvents( void ) const
        { return currentEvents_; }

        //! current path list
        void _setCurrentPathList( const Path::List& value )
        { currentPath_ = value; }

        //! current path list
        const Path::List& _currentPathList( void ) const
        { return currentPath_; }

        //! update current path list
        void _updateCurrentPathList( void )
        { _setCurrentPathList( _pathList( _currentEvents() ) ); }

        //! paint current path list
        void _paintCurrentPathList( QPainter& painter )
        { _paintPathList( painter, _currentPathList(), true ); }

        //@}

        //!@name selected events
        //@{

        //! update selection to include given position
        void _updateSelection( const QPoint&, CalendarEvent::SelectionMode = CalendarEvent::SelectionRange );

        //! store list of selected month events
        void _updateSelectedDays( const QDate&, const CalendarEvent::SelectionMode&, const bool& = false );

        //! clear selected events
        void _clearSelectedDays( void )
        { _updateSelectedDays( QDate(), CalendarEvent::SelectionRange, true ); }

        //! selected events
        const CalendarEvent::List& _selectedDays( void ) const
        { return selectedDays_; }

        //! selected date
        void _setSelectedDate( const QDate& value )
        { selectedData_ = value; }

        //! selected date
        const QDate& _selectedDate( void ) const
        { return selectedData_; }

        //! selected path list
        void _setSelectedPathList( const Path::List& value )
        { selectedPathList_ = value; }

        //! selected path list
        const Path::List& _selectedPathList( void ) const
        { return selectedPathList_; }

        //! update selected path list
        void _updateSelectedPathList( void )
        { _setSelectedPathList( _pathList( _selectedDays() ) ); }

        //! paint selected path list
        void _paintSelectedPathList( QPainter& painter )
        { _paintPathList( painter, _selectedPathList(), false ); }

        //@}


        //!@name geometry
        //@{

        //! update geometry
        void _updateGeometry( void );

        //! rows
        const int& _nRows( void ) const
        { return nRows_; }

        //! columns
        const int& _nColumns( void ) const
        { return nColumns_; }

        //! cell width
        const qreal& _cellWidth( void ) const
        { return cellWidth_; }

        //! cell height
        const qreal& _cellHeight( void ) const
        { return cellHeight_; }

        //! cell size
        const QSizeF _cellSize( void ) const
        { return QSizeF( cellWidth_, cellHeight_ ); }

        //@}

        //! get path list associated to events
        Path::List _pathList( const CalendarEvent::List& ) const;

        //! paint events
        void _paintPathList( QPainter&, const Path::List&, const bool& ) const;

        //! paint horizontal line
        void _paintSeparator( QPainter& ) const;

        //! paint text
        void _paintText( QPainter& ) const;

        //! current date
        bool _setDate( const QDate& );

        //! restart timer for auto-update of current day
        void _restartTimer( void );

        //! update widget tooltip, if any
        virtual bool _updateToolTip( const QString& );

        //! notifications
        void _checkNotifications( void );

        protected Q_SLOTS:

        //! configuration
        virtual void _configuration( void );

        //! update configuration
        virtual void _updateConfiguration( void );

        //! adjust time
        virtual void _adjustTime( void );

        //! previous month
        virtual void _selectPreviousMonth( void )
        {
            if( _setDate( date_.addMonths( -1 ) ) )
            { update(); }
        }

        //! previous year
        virtual void _selectPreviousYear( void )
        {
            if( _setDate( date_.addYears( -1 ) ) )
            { update(); }
        }

        //! next month
        virtual void _selectNextMonth( void )
        {
            if( _setDate( date_.addMonths( 1 ) ) )
            { update(); }
        }

        //! next year
        virtual void _selectNextYear( void )
        {
            if( _setDate( date_.addYears( 1 ) ) )
            { update(); }
        }

        //! show current month
        void _selectCurrentMonth( void )
        {
            if( _setDate( QDate::currentDate() ) )
            { update(); }
        }

        //! select month from dialog
        virtual void _selectMonthFromDialog( void );

        //! edit event
        virtual void _editEvent( void );

        //! remove event
        virtual void _removeEvent( void );

        //! event selection
        virtual void _addEvent( void );

        //! configure events
        virtual void _configureEvents( void );

        //! toggle showing of past events
        virtual void _toggleShowPastEvents( bool );

        //! toggle showing of week numbers
        virtual void _toggleShowWeekNumbers( bool );

        //! toggle showing of week numbers
        virtual void _toggleShowInvalidDays( bool );

        //! event notification sent
        virtual void _eventNotificationSent( const CalendarEvent& );

        private:

        //! install actions
        void _installActions( void );

        //!@name actions
        //@{

        //! previous month
        QAction* previousMonthAction_;

        //! previous year
        QAction* previousYearAction_;

        //! next month
        QAction* nextMonthAction_;

        //! next year
        QAction* nextYearAction_;

        //! current month action
        QAction* currentMonthAction_;

        //! select action
        /* used to select month, year by date */
        QAction* selectMonthAction_;

        //! edit event
        QAction* editEventAction_;

        //! remove event
        QAction* removeEventAction_;

        //! event selection
        QAction* addEventAction_;

        //! add event
        QAction* configureEventsAction_;

        //! adjust time
        QAction* adjustTimeAction_;

        //! past events
        QAction* showPastEventsAction_;

        //! week numbers
        QAction* showWeekNumbersAction_;

        //! invalid days
        QAction* showInvalidDaysAction_;

        //@}

        //!@name properties
        //@{

        //! foreground color
        Color currentDayColor_;

        //! foreground color
        Color markedDayColor_;

        //@}

        //!@name geometry
        //@{

        //! columns
        int nColumns_;

        //! rows
        int nRows_;

        //! cell width
        qreal cellWidth_;

        //! cell height
        qreal cellHeight_;

        //! cells
        Cell::List cells_;

        //! week cells
        Cell::List weekCells_;

        //! week days
        Cell::List weekDayCells_;

        //! used to sort rectangles
        class SortRectFTor
        {
            public:

            bool operator() ( const QRectF& first, const QRectF& second ) const
            {
                return
                    first.top() < second.top() ||
                    ( first.top() == second.top() && first.left() < second.left() );
            }

        };

        //@}

        //! redraw timer
        QBasicTimer timer_;

        //! header
        CalendarHeader* header_;

        //! calendar event list
        CalendarEventList events_;

        //! calendar current event list
        CalendarEvent::List currentEvents_;

        //! current path list
        Path::List currentPath_;

        //! current event, if any
        CalendarEvent currentEvent_;

        //! selected date
        QDate selectedData_;

        //! calendate selected events
        CalendarEvent::List selectedDays_;

        //! current event, if any
        CalendarEvent selectedEvent_;

        //! selected path list
        Path::List selectedPathList_;

        //! displayed date
        QDate date_;

    };

};

#endif
