// $Id: AlarmDialog.cpp,v 1.39 2014/04/28 08:43:11 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlarmDialog.h"
#include "AlarmDialog.moc"

#include "Color.h"
#include "CompositeEngine.h"
#include "Debug.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "SvgEngine.h"
#include "XmlOptions.h"
#include "XcbUtil.h"

#include <QFrame>
#include <QLayout>
#include <QLabel>
#include <QPainter>
#include <QToolButton>

namespace Clock
{
    //_______________________________________________________
    AlarmDialog::AlarmDialog( QWidget* parent, const Alarm& alarm ):
        Transparency::TransparentWidget( parent, Qt::Tool | Qt::FramelessWindowHint ),
        useSvg_( false ),
        alarm_( alarm ),
        button_( Qt::NoButton )
    {

        Debug::Throw( "AlarmDialog::AlarmDialog.\n" );
        setAttribute( Qt::WA_DeleteOnClose );
        setWindowTitle( tr( "Alarm - Clock" ) );

        QVBoxLayout *mainLayout = new QVBoxLayout();
        mainLayout->setMargin(10);
        mainLayout->setSpacing(5);
        setLayout( mainLayout );

        // svg
        useSvg_ = XmlOptions::get().get<bool>( "USE_SVG" );

        // update foreground color
        QColor foregroundColor( XmlOptions::get().get<Base::Color>( "TRANSPARENCY_FOREGROUND_COLOR" ) );
        QPalette palette( AlarmDialog::palette() );
        if( foregroundColor.isValid() )
        {
            palette.setColor( QPalette::Text, foregroundColor );
            palette.setColor( QPalette::WindowText, foregroundColor );
            setPalette( palette );
        }

        // icon
        QVBoxLayout *vLayout( new QVBoxLayout() );
        vLayout->setMargin(10);
        vLayout->setSpacing(5);

        QHBoxLayout* hLayout( new QHBoxLayout() );
        hLayout->setMargin(0);
        hLayout->setSpacing(10);
        mainLayout->addLayout( hLayout );
        QLabel* label = new QLabel( this );
        label->setMargin(10);
        label->setPixmap( IconEngine::get( IconNames::AlarmLarge ).pixmap( IconSize( IconSize::Maximum ) ) );
        hLayout->addWidget( label, 0, Qt::AlignTop );
        hLayout->addLayout( vLayout );

        // text
        vLayout->addStretch();

        // alarm time
        vLayout->addWidget( label = new QLabel( alarm.time().toString( TimeStamp::Time ), this ), Qt::AlignLeft|Qt::AlignTop );
        QFont font( label->font() );
        font.setWeight( QFont::Bold );
        font.setPointSize( 20 );
        label->setFont( font );

        // alarm text
        vLayout->addWidget( new QLabel( alarm.text(), this ), Qt::AlignLeft|Qt::AlignTop );
        vLayout->addStretch();

        QFrame* frame = new QFrame( this );
        frame->setFrameStyle( QFrame::HLine | QFrame::Plain );
        if( useSvg_ ) frame->setPalette( palette );
        layout()->addWidget( frame );

        // button layout
        QBoxLayout* buttonLayout = new QBoxLayout( QBoxLayout::LeftToRight );
        buttonLayout->setSpacing(5);
        buttonLayout->setMargin(0);
        buttonLayout->addStretch( 1 );
        mainLayout->addLayout( buttonLayout, 0 );

        {
            // repeat button
            QToolButton* button;
            buttonLayout->addWidget( button = new QToolButton( this ) );
            button->setIcon( IconEngine::get( IconNames::Reload ) );
            button->setText( tr( "Repeat" ) );
            button->setAutoRaise( true );
            button->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
            button->setIconSize( IconSize( IconSize::Medium ) );
            if( useSvg_ ) button->setPalette( palette );

            connect( button, SIGNAL(clicked()), SLOT(_repeat()) );

        }

        {
            // close button
            QToolButton* button;
            buttonLayout->addWidget( button = new QToolButton( this ) );
            button->setIcon( IconEngine::get( IconNames::DialogClose ) );
            button->setText( tr( "Close" ) );
            button->setAutoRaise( true );
            button->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
            button->setIconSize( IconSize( IconSize::Medium ) );
            if( useSvg_ ) button->setPalette( palette );

            connect( button, SIGNAL(clicked()), SLOT(_close()) );
        }

        adjustSize();
        return;

    }

    //___________________________________________________
    void AlarmDialog::mousePressEvent( QMouseEvent *event )
    {

        Transparency::TransparentWidget::mousePressEvent( event );

        _setButton( event->button() );
        if( _button() == Qt::LeftButton )
        {

            clickPosition_ = event->pos() + QPoint(geometry().topLeft() - frameGeometry().topLeft());

            #if HAVE_X11
            // try use X11 Window Manager to handle event
            if( XcbUtil::get().moveResizeWidget( this, event->globalPos(), X11Defines::_NET_WM_MOVERESIZE_MOVE ) ) return;
            else setCursor( Qt::SizeAllCursor );
            #else
            setCursor( Qt::SizeAllCursor );
            #endif
        }

    }

    //___________________________________________________
    void AlarmDialog::mouseReleaseEvent( QMouseEvent* )
    {
        _setButton( Qt::NoButton );
        unsetCursor();
    }

    //___________________________________________________
    void AlarmDialog::mouseMoveEvent( QMouseEvent *event )
    {

        if( _button() == Qt::LeftButton )
        {

            QPoint global_point( event->globalPos() - clickPosition_ );
            move( global_point );

        }

    }

    //__________________________________________________
    void AlarmDialog::_paint( QPaintDevice& device, const QRect& rect )
    {

        QPainter painter( &device );
        painter.setClipRect( rect );
        painter.setPen( palette().color( QPalette::Text ) );

        if( Transparency::CompositeEngine::get().isAvailable() )
        {
            painter.setRenderHints(QPainter::SmoothPixmapTransform);
            painter.setCompositionMode(QPainter::CompositionMode_Source );
        }

        if( useSvg_ && Svg::SvgEngine::get().isValid() )
        { painter.drawPixmap( QPoint(0,0), Svg::SvgEngine::get().get( size() ) ); }
        painter.end();

        // loop over children and render them
        QList<QWidget*> widgets( findChildren<QWidget*>() );
        for( QList<QWidget*>::iterator iter = widgets.begin(); iter != widgets.end(); ++iter )
        { _render( device, **iter, rect ); }

    }


    //________________________________________________
    void AlarmDialog::_render( QPaintDevice& device, QWidget& widget, const QRect& clip )
    { widget.render( &device, widget.geometry().topLeft(), widget.rect().intersected( clip.translated( -widget.geometry().topLeft() ) ), 0 ); }

}
